# frozen_string_literal: true

# rubocop:disable Metrics/BlockLength

require 'nagios_output'

RSpec.describe NagiosOutput do
    it 'has a version number' do
        expect(NagiosOutput::VERSION).not_to be nil
    end

    context 'Class Accessor Method' do
        context 'Test ok()' do
            before(:each) do
                @n = NagiosOutput::Nagios
            end

            context 'With a message' do
                it 'says OK - test 1 2 3' do
                    begin
                        expect(@n.ok('test 1 2 3')).to eql('OK - test 1 2 3')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 0' do
                    begin
                        @n.ok('test 1 2 3')
                    rescue SystemExit => e
                        expect(e.status).to eq(0)
                    end
                end
            end

            context 'Without a message' do
                it 'says nothing at all' do
                    begin
                        expect(@n.ok).to eql
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 0' do
                    begin
                        @n.ok
                    rescue SystemExit => e
                        expect(e.status).to eq(0)
                    end
                end
            end
        end

        context 'Test warning()' do
            before(:each) do
                @n = NagiosOutput::Nagios
            end

            context 'With a message' do
                it 'says WARNING - test 1 2 3' do
                    begin
                        expect(@n.warning('test 1 2 3')).to eql('WARNING - test 1 2 3')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 1' do
                    begin
                        @n.warning('test 1 2 3')
                    rescue SystemExit => e
                        expect(e.status).to eq(1)
                    end
                end
            end

            context 'Without a message' do
                it 'says nothing at all' do
                    begin
                        expect(@n.warning).to eql
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 1' do
                    begin
                        @n.warning
                    rescue SystemExit => e
                        expect(e.status).to eq(1)
                    end
                end
            end
        end

        context 'Test critical()' do
            before(:each) do
                @n = NagiosOutput::Nagios
            end

            context 'With a message' do
                it 'says CRITICAL - test 1 2 3' do
                    begin
                        expect(@n.critical('test 1 2 3')).to eql('CRITICAL - test 1 2 3')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 2' do
                    begin
                        @n.critical('test 1 2 3')
                    rescue SystemExit => e
                        expect(e.status).to eq(2)
                    end
                end
            end

            context 'Without a message' do
                it 'says nothing at all' do
                    begin
                        expect(@n.critical).to eql
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 2' do
                    begin
                        @n.critical
                    rescue SystemExit => e
                        expect(e.status).to eq(2)
                    end
                end
            end
        end

        context 'Test unknown()' do
            before(:each) do
                @n = NagiosOutput::Nagios
            end

            context 'With a message' do
                it 'says UNKNOWN - test 1 2 3' do
                    begin
                        expect(@n.unknown('test 1 2 3')).to eql('UNKNOWN - test 1 2 3')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 3' do
                    begin
                        @n.unknown('test 1 2 3')
                    rescue SystemExit => e
                        expect(e.status).to eq(3)
                    end
                end
            end

            context 'Without a message' do
                it 'says nothing at all' do
                    begin
                        expect(@n.unknown).to eql
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 3' do
                    begin
                        @n.unknown
                    rescue SystemExit => e
                        expect(e.status).to eq(3)
                    end
                end
            end
        end
    end

    context 'String Accessor Method' do
        context 'Test ok()' do
            context 'With a message' do
                it 'says OK - test 1 2 3' do
                    begin
                        expect('test 1 2 3'.nagios_ok).to eql('OK - test 1 2 3')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 0' do
                    begin
                        'test 1 2 3'.nagios_ok
                    rescue SystemExit => e
                        expect(e.status).to eq(0)
                    end
                end
            end

            context 'Without a message' do
                it 'says nothing at all' do
                    begin
                        expect(''.nagios_ok).to eql('')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 0' do
                    begin
                        expect(''.nagios_ok)
                    rescue SystemExit => e
                        expect(e.status).to eq(0)
                    end
                end
            end
        end

        context 'Test warning()' do
            context 'With a message' do
                it 'says WARNING - test 1 2 3' do
                    begin
                        expect('test 1 2 3'.nagios_warning).to eql('WARNING - test 1 2 3')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 1' do
                    begin
                        'test 1 2 3'.nagios_warning
                    rescue SystemExit => e
                        expect(e.status).to eq(1)
                    end
                end
            end

            context 'Without a message' do
                it 'says nothing at all' do
                    begin
                        expect(''.nagios_warning).to eql('')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 1' do
                    begin
                        ''.nagios_warning
                    rescue SystemExit => e
                        expect(e.status).to eq(1)
                    end
                end
            end
        end

        context 'Test critical()' do
            context 'With a message' do
                it 'says CRITICAL - test 1 2 3' do
                    begin
                        expect('test 1 2 3'.nagios_critical).to eql('CRITICAL - test 1 2 3')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 2' do
                    begin
                        'test 1 2 3'.nagios_critical
                    rescue SystemExit => e
                        expect(e.status).to eq(2)
                    end
                end
            end

            context 'Without a message' do
                it 'says nothing at all' do
                    begin
                        expect(''.nagios_critical).to eql('')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 2' do
                    begin
                        ''.nagios_critical
                    rescue SystemExit => e
                        expect(e.status).to eq(2)
                    end
                end
            end
        end

        context 'Test unknown()' do
            context 'With a message' do
                it 'says UNKNOWN - test 1 2 3' do
                    begin
                        expect('test 1 2 3'.nagios_unknown).to eql('UNKNOWN - test 1 2 3')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 3' do
                    begin
                        'test 1 2 3'.nagios_unknown
                    rescue SystemExit => e
                        expect(e.status).to eq(3)
                    end
                end
            end

            context 'Without a message' do
                it 'says nothing at all' do
                    begin
                        expect(''.nagios_unknown).to eql('')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 3' do
                    begin
                        ''.nagios_unknown
                    rescue SystemExit => e
                        expect(e.status).to eq(3)
                    end
                end
            end
        end
    end

    context 'Int Accessor Method' do
        context 'Test ok()' do
            context 'With a message' do
                it 'says OK - 123' do
                    begin
                        expect(123.nagios_ok).to eql('OK - 123')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 0' do
                    begin
                        123.nagios_ok
                    rescue SystemExit => e
                        expect(e.status).to eq(0)
                    end
                end
            end
        end

        context 'Test warning()' do
            context 'With a message' do
                it 'says WARNING - 123' do
                    begin
                        expect(123.nagios_warning).to eql('WARNING - 123')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 1' do
                    begin
                        123.nagios_warning
                    rescue SystemExit => e
                        expect(e.status).to eq(1)
                    end
                end
            end
        end

        context 'Test critical()' do
            context 'With a message' do
                it 'says CRITICAL - 123' do
                    begin
                        expect(123.nagios_critical).to eql('CRITICAL - 123')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 2' do
                    begin
                        123.nagios_critical
                    rescue SystemExit => e
                        expect(e.status).to eq(2)
                    end
                end
            end
        end

        context 'Test unknown()' do
            context 'With a message' do
                it 'says UNKNOWN - 123' do
                    begin
                        expect(123.nagios_unknown).to eql('UNKNOWN - 123')
                    rescue SystemExit => e
                        _actual = e.status
                    end
                end

                it 'the exit value should be 3' do
                    begin
                        123.nagios_unknown
                    rescue SystemExit => e
                        expect(e.status).to eq(3)
                    end
                end
            end
        end
    end

    context 'Test Error Handling' do
        it 'Test invalid set colour type raises an exception direct call' do
            expect { NagiosOutput::Nagios.use_colour('invalid') }.to raise_error(ArgumentError, 'Invalid option: use true or false only')
        end

        it 'Test invalid set colour type raises an exception aliases call' do
            expect { NagiosOutput::Nagios.use_color('invalid') }.to raise_error(ArgumentError, 'Invalid option: use true or false only')
        end
    end
end

# rubocop:enable Metrics/BlockLength
